/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.
 */

#include "config/option/hostportoption.h"

#include <string>
#include <vector>

#include <boost/foreach.hpp>

#include <logfunc.hpp>

#include "boost_assert_handler.h"

using namespace std;
using boost::program_options::value;
using boost::program_options::variables_map;
using I2n::Logger::GlobalLogger;

//-----------------------------------------------------------------------------
// HostPortOption
//-----------------------------------------------------------------------------

HostPortOption::HostPortOption() :
    HostConfigurationOption(
        "host.port",
        value< vector<int> >(),//->default_value( 80 ), // HTTP port;,
        "Host port number."
    )
{
}

HostPortOption::~HostPortOption()
{
}

bool HostPortOption::parse(
        const variables_map& vm,
        Configuration *configuration
)
{
    size_t hosts_count = 0;
    size_t hosts_ports_count = 0;
    bool parsed_success = false;

    // [host] port
    if ( 1 <= vm.count( get_command_string() ) )
    {
        HostList hosts_list = configuration->get_hosts();
        HostList::iterator hosts_list_iterator = hosts_list.begin();
        hosts_count = hosts_list.size();

        vector<int> hosts_ports_list = vm[ get_command_string() ].as< vector<int> >();
        hosts_ports_count = hosts_ports_list.size();

        BOOST_ASSERT( hosts_count >= hosts_ports_count );

        BOOST_FOREACH( int host_port, hosts_ports_list )
        {
            BOOST_ASSERT( ( 0 <= host_port ) && ( host_port <= numeric_limits<uint16_t>::max() ) );

            HostItem host_item = *hosts_list_iterator;
            host_item->set_port( static_cast<uint16_t>(host_port) );
            ++hosts_list_iterator;

            GlobalLogger.info() << get_command_string() << "=" << host_port << endl;
        }

        parsed_success = true;
    }

    set_hosts_count( hosts_ports_count );

    BOOST_ASSERT( hosts_count == hosts_ports_count );

    return parsed_success;
}
